#include <ReWindow/types/Window.h>
#include <ReWindow/data/WindowsEventLoop.h>
#include <GL/gl.h>

class RWindow::Platform {
public:
    HINSTANCE hInstance;
    HWND hwnd;
    HDC hdc;
};

using namespace ReWindow;

HGLRC glContext;

bool OpenGLWindow::Open() {
  platform->hInstance = GetModuleHandle(nullptr);

  WNDCLASS wc = { };
  wc.lpfnWndProc = WindowProc;
  wc.hInstance = platform->hInstance;
  wc.lpszClassName = "RWindowClass";
  RegisterClass(&wc);
  platform->hwnd = CreateWindowEx(
      0,
      "RWindowClass",
      title.c_str(),
      WS_OVERLAPPEDWINDOW,
      CW_USEDEFAULT, CW_USEDEFAULT, width, height,
      nullptr,
      nullptr,
      platform->hInstance,
      nullptr
  );
    SetWindowLongPtr(platform->hwnd, GWLP_USERDATA, reinterpret_cast<LONG_PTR>(this));
  PIXELFORMATDESCRIPTOR pfd = {
      sizeof(PIXELFORMATDESCRIPTOR),
      1,
      PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER,
      PFD_TYPE_RGBA,
      24,
      0, 0, 0, 0, 0, 0,
      0,
      0,
      0,
      0, 0, 0, 0,
      24,
      8,
      0,
      PFD_MAIN_PLANE,
      0,
      0, 0, 0
  };

  platform->hdc = GetDC(platform->hwnd);
  int pixelFormat = ChoosePixelFormat(platform->hdc, &pfd);
  SetPixelFormat(platform->hdc, pixelFormat, &pfd);
  glContext = wglCreateContext(platform->hdc);
  wglMakeCurrent(platform->hdc, glContext);
  ShowWindow(platform->hwnd, SW_SHOW);
  open = true;

    // TODO don't link directly to OpenGL.
    return true;
}

std::string OpenGLWindow::GetGraphicsDriverVendor() {
    return std::string(reinterpret_cast<const char*>(glGetString(GL_VENDOR)));
}

void OpenGLWindow::SwapBuffers() {
    ::SwapBuffers(platform->hdc);
}

void OpenGLWindow::SetVsyncEnabled(bool b) {
    typedef BOOL(WINAPI* PFNWGLSWAPINTERVALEXTPROC)(int interval);
    auto wglSwapIntervalEXT = (PFNWGLSWAPINTERVALEXTPROC) wglGetProcAddress("wglSwapIntervalEXT");

    if (wglSwapIntervalEXT)
        wglSwapIntervalEXT(b ? 1 : 0);
}

OpenGLWindow::OpenGLWindow(const std::string& title, int width, int height, uint8_t gl_major, uint8_t gl_minor)
: gl_major(gl_major), gl_minor(gl_minor), RWindow(title, width, height) {}

